#ifdef WIN32
#include <windows.h>
#endif


#include <GL/glew.h>
#include <GL/glut.h>
// Headers for DevIL - a library for loading images of many filetypes
#include <IL/il.h>
#include <IL/ilut.h>

#include <cstdlib>

#include "glutil.h"
#include "float4x4.h"

using namespace chag;


// The shaderProgram holds the vertexShader and fragmentShader
GLuint shaderProgram;

// The vertexArrayObject here will hold the pointers to 
// the vertex data (in vertBuffer) and color data per vertex (in colorBuffer)
GLuint		vertBuffer, colorBuffer, indexBuffer, vertexArrayObject;						



void initGL()
{
	//************************************
	//			Load Extensions
	//************************************
	glewInit();					// initialize glew, which provides easy access to OpenGL functions newer than version 1.1
	ilInit();					// initialize devIL (developers Image Library)
	ilutRenderer(ILUT_OPENGL);  // initialize devIL

	// Workaround for AMD, which hopefully will not be neccessary in the near future...
	if (!glBindFragDataLocation)
	{
		glBindFragDataLocation = glBindFragDataLocationEXT;
	}

	///////////////////////////////////////////////////////////////////////////
	// Create the floor quad
	///////////////////////////////////////////////////////////////////////////	
	// Vertex positions
	const float verts[] = {
		// X Y Z
		-10.0f,   -10.0f, -30.0f,    // v0
		-10.0f,   -10.0f, -330.0f,   // v1
		 10.0f,   -10.0f, -330.0f,   // v2
		 10.0f,   -10.0f, -30.0f     // v3
	};

	// Vertex colors
	const float colors[] = {
		//  R     G		B
		1.0f, 1.0f, 1.0f,		// White
		0.5f, 0.5f, 0.8f,		// Lightblue
		0.5f, 0.5f, 0.8f,		// Lightblue
		1.0f, 1.0f, 1.0f		// White
	};


	const int indices[] = {
		0,1,3, // Triangle 1
		1,2,3  // Triangle 2
	};

	// Create the buffer objects
	glGenBuffers( 1, &vertBuffer );															// Create a handle for the vertex position buffer
	glBindBuffer( GL_ARRAY_BUFFER, vertBuffer );											// Set the newly created buffer as the current one
	glBufferData( GL_ARRAY_BUFFER, sizeof(verts), verts, GL_STATIC_DRAW );			// Send the vetex position data to the current buffer

	glGenBuffers( 1, &colorBuffer );															// Create a handle for the vertex color buffer
	glBindBuffer( GL_ARRAY_BUFFER, colorBuffer );										// Set the newly created buffer as the current one
	glBufferData( GL_ARRAY_BUFFER, sizeof(colors), colors, GL_STATIC_DRAW );			// Send the color data to the current buffer

	CHECK_GL_ERROR();

	glGenBuffers( 1, &indexBuffer );
	glBindBuffer( GL_ELEMENT_ARRAY_BUFFER, indexBuffer );										
	glBufferData( GL_ELEMENT_ARRAY_BUFFER, sizeof(indices), indices, GL_STATIC_DRAW );			

	// Create the vertex array object
	glGenVertexArrays(1, &vertexArrayObject);
	glBindVertexArray(vertexArrayObject);
	CHECK_GL_ERROR();
	glBindBuffer( GL_ARRAY_BUFFER, vertBuffer );	
	glVertexAttribPointer(0, 3, GL_FLOAT, false/*normalized*/, 0/*stride*/, 0/*offset*/ );	
	glBindBuffer( GL_ARRAY_BUFFER, colorBuffer );
	glVertexAttribPointer(1, 3, GL_FLOAT, false/*normalized*/, 0/*stride*/, 0/*offset*/ );
	glBindBuffer( GL_ELEMENT_ARRAY_BUFFER, indexBuffer );
	glEnableVertexAttribArray(0);
	glEnableVertexAttribArray(1);
	CHECK_GL_ERROR();


	// The loadShaderProgram and linkShaderProgam functions are defined in glutil.cpp and 
	// do exactly what we did in lab1 but are hidden for convenience
	shaderProgram = loadShaderProgram("simple.vert", "simple.frag"); 
	glBindAttribLocation(shaderProgram, 0, "vertex"); 	
	glBindAttribLocation(shaderProgram, 1, "color");
	glBindFragDataLocation(shaderProgram, 0, "fragmentColor");
	linkShaderProgram(shaderProgram); 
	//**********************************************

	//************************************
	//			Load Texture
	//************************************



	CHECK_GL_ERROR();
}

void display(void)
{
	glClearColor(0.2,0.2,0.8,1.0);						// Set clear color
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); // Clears the color buffer and the z-buffer

	int w = glutGet((GLenum)GLUT_WINDOW_WIDTH);
	int h = glutGet((GLenum)GLUT_WINDOW_HEIGHT);
	glViewport(0, 0, w, h);								// Set viewport

	// We disable backface culling for this tutorial, otherwise care must be taken with the winding order
	// of the vertices. It is however a lot faster to enable culling when drawing large scenes.
	glDisable(GL_CULL_FACE);
	// Disable depth testing
	glDisable(GL_DEPTH_TEST);
	// Shader Program
	glUseProgram( shaderProgram );				// Set the shader program to use for this draw call

	// Set up a projection matrix
	float4x4 projectionMatrix = perspectiveMatrix(45.0f, float(w)/float(h), 0.01f, 300.0f); 
	// Send it to the vertex shader
	int loc = glGetUniformLocation(shaderProgram, "projectionMatrix");
	glUniformMatrix4fv(loc, 1, false, &projectionMatrix.c1.x);


	glBindVertexArray(vertexArrayObject);
	glDrawElements(GL_TRIANGLES, 6, GL_UNSIGNED_INT, 0);


	glUseProgram( 0 );						// "unsets" the current shader program. Not really necessary.
	CHECK_GL_ERROR();

	glutSwapBuffers();  // swap front and back buffer. This frame will now be displayed.
}

int main(int argc, char *argv[])
{
	glutInit(&argc, argv);
	/* open window of size 800x600 with double buffering, RGB colors, and Z-buffering */
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowSize(512,512);
	glutCreateWindow("OpenGL Lab 2");
	/* the display function is called once when the gluMainLoop is called,
	* but also each time the window has to be redrawn due to window 
	* changes (overlap, resize, etc).  
	*/
	glutDisplayFunc(display);	// Set the main redraw function

	initGL();

	glutMainLoop();  /* start the program main loop */

	return 0;          
}
